﻿using Celeste.Mod.Entities;
using IL.Celeste;
using Microsoft.Xna.Framework;
using Monocle;
using System;
using System.Collections.Generic;
using System.Linq;

namespace Celeste.Mod.Mine.Entities
{
    [Tracked(false)]
    public class CrystalBall : Entity
    {
        public enum ShotPatterns
        {
            Single,
            Double,
            Triple
        }

        public static ParticleType P_Trail;

        private const float MoveSpeed = 100f;

        private const float CantKillTime = 0.15f;

        private const float AppearTime = 0.1f;

        private CrystalShooter source;

        private Level level;

        private Vector2 speed;

        private float particleDir;

        private Vector2 anchor;

        private Vector2 perp;

        private Player target;

        private Vector2 targetPt;

        private float angleOffset;

        private bool dead;

        private float cantKillTimer;

        private float appearTimer;
        private float sizeHitbox = 8;
        private bool hasBeenInCamera;

        private SineWave sine;

        private float sineMult;
        private static SpriteBank spriteBank;
        private Sprite sprite;

        public static void LoadContent()
        {
            spriteBank = new SpriteBank(GFX.Game, "Graphics/Mine/Sprites.xml");
        }

        public CrystalBall()
            : base(Vector2.Zero)
        {
            Add(sprite = spriteBank.Create("CrystalProjectile"));
            base.Collider = new Hitbox(sizeHitbox, sizeHitbox, -(sizeHitbox / 2), -(sizeHitbox / 2));
            Add(new PlayerCollider(OnPlayer));
            base.Depth = -1000000;
            Add(sine = new SineWave(1.4f));
        }

        public CrystalBall Init(CrystalShooter source, Player target, float angleOffset = 0f)
        {
            this.source = source;
            anchor = (Position = source.Center);
            this.target = target;
            this.angleOffset = angleOffset;
            dead = (hasBeenInCamera = false);
            cantKillTimer = 0.15f;
            appearTimer = 0.1f;
            sine.Reset();
            sineMult = 0f;
            sprite.Play("charge", restart: true);
            InitSpeed();
            return this;
        }

        public CrystalBall Init(CrystalShooter source, Vector2 target)
        {
            this.source = source;
            anchor = (Position = source.Center);
            this.target = null;
            angleOffset = 0f;
            targetPt = target;
            dead = (hasBeenInCamera = false);
            cantKillTimer = 0.15f;
            appearTimer = 0.1f;
            sine.Reset();
            sineMult = 0f;
            sprite.Play("charge", restart: true);
            InitSpeed();
            return this;
        }

        private void InitSpeed()
        {
            if (target != null)
            {
                speed = (target.Center - base.Center).SafeNormalize(100f);
            }
            else
            {
                speed = (targetPt - base.Center).SafeNormalize(100f);
            }
            if (angleOffset != 0f)
            {
                speed = speed.Rotate(angleOffset);
            }
            perp = speed.Perpendicular().SafeNormalize();
            particleDir = (-speed).Angle();
        }

        public override void Added(Scene scene)
        {
            base.Added(scene);
            level = SceneAs<Level>();
        }

        public override void Removed(Scene scene)
        {
            base.Removed(scene);
            level = null;
        }

        public override void Update()
        {
            Level l = Scene as Level;
            base.Update();
            if (appearTimer > 0f)
            {
                Position = (anchor = source.Position);
                appearTimer -= Engine.DeltaTime;

                return;
            }

            if (cantKillTimer > 0f)
            {
                cantKillTimer -= Engine.DeltaTime;
            }
            anchor += (speed * Engine.DeltaTime) * 2.5f;
            Position = anchor + perp * sineMult * sine.Value * 3f;
            sineMult = Calc.Approach(sineMult, 1f, Engine.DeltaTime);
            if (!dead)
            {
                foreach (SeekerBarrier s in l.Entities.FindAll<SeekerBarrier>())
                {
                    if (CollideRect(new Rectangle((int)s.Collider.AbsoluteX, (int)s.Collider.AbsoluteY, (int)s.Collider.Width, (int)s.Collider.Height)))
                    {
                        for (int i = 0; i < 100; i++)
                        {
                            level.ParticlesFG.Emit(P_Trail, 2, base.Center, Vector2.One * 4f, i);
                        }
                        Destroy();
                    }
                }
                foreach (TouchSwitch s in l.Entities.FindAll<TouchSwitch>())
                {
                    if (CollideRect(new Rectangle((int)s.Collider.AbsoluteX, (int)s.Collider.AbsoluteY, (int)s.Collider.Width, (int)s.Collider.Height)))
                    {
                        //s.Switch.Activate();
                        s.TurnOn();
                    }
                }
                level.ParticlesFG.Emit(P_Trail, 2, base.Center, Vector2.One * 2f, particleDir);
                bool inCamera = level.IsInCamera(Position, 8f);
                if (inCamera && !hasBeenInCamera)
                {
                    hasBeenInCamera = true;
                }
                else if (!inCamera && hasBeenInCamera)
                {
                    Destroy();
                }
                if (base.Scene.OnInterval(0.04f))
                {
                }
            }
        }

        public override void Render()
        {
            Color col = sprite.Color;
            Vector2 pos = sprite.Position;
            sprite.Color = Color.Black;
            sprite.Position = pos + new Vector2(-1f, 0f);
            sprite.Render();
            sprite.Position = pos + new Vector2(1f, 0f);
            sprite.Render();
            sprite.Position = pos + new Vector2(0f, -1f);
            sprite.Render();
            sprite.Position = pos + new Vector2(0f, 1f);
            sprite.Render();
            sprite.Color = col;
            sprite.Position = pos;
            base.Render();
        }

        public void Destroy()
        {
            dead = true;
            RemoveSelf();
        }

        private void OnPlayer(Player player)
        {
            if (!dead)
            {
                player.Die((player.Center - Position).SafeNormalize());
            }
        }

        private void OnBarrier(SeekerBarrier player)
        {
            Destroy();
        }
    }
}