﻿using System;
using System.Collections;
using System.Collections.Generic;
using Celeste;
using Celeste.Mod;
using Microsoft.Xna.Framework;
using Monocle;
using Celeste.Mod.Entities;

namespace Celeste.Mod.Mine.Entities
{
    [Tracked(false)]
    [CustomEntity("Mine/GCrystal")]
    public class GCrystalStaticSpinner : Entity
    {
        public float fireMode = 0;

        private class CoreModeListener : Component
        {
            public GCrystalStaticSpinner Parent;

            public CoreModeListener(GCrystalStaticSpinner parent)
                : base(active: true, visible: false)
            {
                Parent = parent;
            }

            public override void Update()
            {
                Level level = base.Scene as Level;
                if ((Parent.color == CrystalColor.Blue && level.CoreMode == Session.CoreModes.Hot) || (Parent.color == CrystalColor.Red && level.CoreMode == Session.CoreModes.Cold))
                {
                    if (Parent.color == CrystalColor.Blue)
                    {
                        Parent.color = CrystalColor.Red;
                    }
                    else
                    {
                        Parent.color = CrystalColor.Blue;
                    }
                    Parent.ClearSprites();
                    Parent.CreateSprites();
                }
            }
        }

        private class Border : Entity
        {
            private Entity[] drawing = new Entity[2];

            public Border(Entity parent, Entity filler)
            {
                drawing[0] = parent;
                drawing[1] = filler;
                base.Depth = parent.Depth + 2;
            }

            public override void Render()
            {
                if (drawing[0].Visible)
                {
                    DrawBorder(drawing[0]);
                    DrawBorder(drawing[1]);
                }
            }

            private void DrawBorder(Entity entity)
            {
                if (entity != null)
                {
                    foreach (Component component in entity.Components)
                    {
                        Image img = component as Image;
                        if (img != null)
                        {
                            Color col = img.Color;
                            Vector2 pos = img.Position;
                            img.Color = Color.Black;
                            img.Position = pos + new Vector2(0f, -1f);
                            img.Render();
                            img.Position = pos + new Vector2(0f, 1f);
                            img.Render();
                            img.Position = pos + new Vector2(-1f, 0f);
                            img.Render();
                            img.Position = pos + new Vector2(1f, 0f);
                            img.Render();
                            img.Color = col;
                            img.Position = pos;
                        }
                    }
                }
            }
        }

        public static ParticleType P_Move;

        public const float ParticleInterval = 0.02f;

        public bool AttachToSolid;
        public int marging = 500;
        private Entity filler;

        private Border border;

        private float offset = Calc.Random.NextFloat();
        public int seed = 101010;
        private Random rnd;
        private bool expanded;

        private int randomSeed;

        private CrystalColor color;

        public GCrystalStaticSpinner(Vector2 position, bool attachToSolid, CrystalColor color)
            : base(position)
        {
            this.color = CrystalColor.Purple;
            base.Tag = Tags.TransitionUpdate;
            base.Collider = new ColliderList(new Circle(6f), new Hitbox(16f, 4f, -8f, -3f));
            Visible = false;
            seed = new Random((int)(position.X + position.Y)).Range(-1000, 1000);
            Add(new PlayerCollider(OnPlayer));
            Add(new HoldableCollider(OnHoldable));
            Add(new LedgeBlocker());
            base.Depth = -8500;
            AttachToSolid = attachToSolid;
            if (attachToSolid)
            {
                Add(new StaticMover
                {
                    OnShake = OnShake,
                    SolidChecker = IsRiding,
                    OnDestroy = base.RemoveSelf
                });
            }
            randomSeed = Calc.Random.Next();
        }

        public GCrystalStaticSpinner(EntityData data, Vector2 offset)
            : this(data.Position + offset, true, CrystalColor.Purple)
        {
        }

        public override void Awake(Scene scene)
        {
            base.Awake(scene);
            if ((scene as Level).Session.Area.ID == 9)
            {
                Add(new CoreModeListener(this));
                if ((scene as Level).CoreMode == Session.CoreModes.Cold)
                {
                    color = CrystalColor.Blue;
                }
                else
                {
                    color = CrystalColor.Red;
                }
            }
            if (InView())
            {
                CreateSprites();
            }
        }

        public void ForceInstantiate()
        {
            CreateSprites();
            Visible = true;
        }

        public override void Update()
        {
            if (!Visible)
            {
                Collidable = false;
                if (InView())
                {
                    Visible = true;
                    if (!expanded)
                    {
                        CreateSprites();
                    }
                    UpdateHue();
                    if (color == CrystalColor.Rainbow)
                    {
                    }
                }
            }
            else
            {
                base.Update();
                UpdateHue();
                if (color == CrystalColor.Rainbow && base.Scene.OnInterval(0.08f, offset))
                {
                    UpdateHue();
                }
                if (base.Scene.OnInterval(0.25f, offset) && !InView())
                {
                    Visible = false;
                }
                if (base.Scene.OnInterval(0.05f, offset))
                {
                    Player player = base.Scene.Tracker.GetEntity<Player>();
                    if (player != null)
                    {
                        Collidable = (Math.Abs(player.X - base.X) < 128f && Math.Abs(player.Y - base.Y) < 128f);
                    }
                }
            }
            if (filler != null)
            {
                filler.Position = Position;
            }
        }

        private void UpdateHue()
        {
            foreach (Component component in base.Components)
            {
                Image img2 = component as Image;
                if (img2 != null)
                {
                    rnd = new Random((seed));

                    img2.Color.G = GetHue(Position + img2.Position).G;
                    img2.Color.R = GetHue(Position + img2.Position).G;
                    img2.Color.B = GetHue(Position + img2.Position).G;
                    if (fireMode != 0)
                    {
                        img2.Color.R = (byte)Calc.Min(255, Calc.Max(GetHue(Position + img2.Position).G, ((img2.Color.R + GetHue(Position + img2.Position).R) * fireMode)));
                        img2.Color.G = (byte)Math.Max(img2.Color.G, 255 - (int)img2.Color.R);
                        img2.Color.B = (byte)Math.Max(img2.Color.B, 255 - (int)img2.Color.R);
                    }
                    //if (rnd.Range(0, 10) < 2)
                    //{
                    //    if (rnd.Chance(0.33F))
                    //    {
                    //        img2.Color.G = (byte)((img2.Color.G + GetHue(Position + img2.Position).G) / 2);
                    //    }
                    //    if (rnd.Chance(0.33F))
                    //    {
                    //        img2.Color.B = (byte)((img2.Color.B + GetHue(Position + img2.Position).B) / 2);
                    //    }
                    //    if (rnd.Chance(0.33F))
                    //    {
                    //        img2.Color.R = (byte)((img2.Color.R + GetHue(Position + img2.Position).R) / 2);
                    //    }
                    //}
                    //else
                    //{
                    //}
                }
            }
            if (filler != null)
            {
                foreach (Component component2 in filler.Components)
                {
                    Image img = component2 as Image;
                    if (img != null)
                    {
                        img.Color.G = GetHue(Position + img.Position).G;
                        img.Color.R = GetHue(Position + img.Position).G;
                        img.Color.B = GetHue(Position + img.Position).G;
                        if (fireMode != 0)
                        {
                            img.Color.R = (byte)Calc.Min(Calc.Max(GetHue(Position + img.Position).G, ((img.Color.R + GetHue(Position + img.Position).R) * fireMode)), 255);
                            img.Color.G = (byte)Math.Max(img.Color.G, 255 - (int)img.Color.R);
                            img.Color.B = (byte)Math.Max(img.Color.B, 255 - (int)img.Color.R);
                        }
                        rnd = new Random((seed));
                        //if (rnd.Range(0, 10) < 2)
                        //{
                        //    if (rnd.Chance(0.33F))
                        //    {
                        //        img.Color.G = (byte)((img.Color.G + GetHue(Position + img.Position).G) / 2);
                        //    }
                        //    if (rnd.Chance(0.33F))
                        //    {
                        //        img.Color.B = (byte)((img.Color.B + GetHue(Position + img.Position).B) / 2);
                        //    }
                        //    if (rnd.Chance(0.33F))
                        //    {
                        //        img.Color.R = (byte)((img.Color.R + GetHue(Position + img.Position).R) / 2);
                        //    }
                        //}
                        //else
                        //{
                        //    img.Color.G = GetHue(Position + img.Position).G;
                        //    img.Color.R = GetHue(Position + img.Position).G;
                        //    img.Color.B = GetHue(Position + img.Position).G;
                        //}
                    }
                }
            }
        }

        private bool InView()
        {
            Camera camera = (base.Scene as Level).Camera;
            if (base.X > camera.X - 16f && base.Y > camera.Y - 16f && base.X < camera.X + 320f + 16f)
            {
                return base.Y < camera.Y + 180f + 16f;
            }
            return false;
        }

        private void CreateSprites()
        {
            if (!expanded)
            {
                Calc.PushRandom(randomSeed);
                List<MTexture> imgs = GFX.Game.GetAtlasSubtextures("danger/crystal/fg_mine");
                MTexture img = Calc.Random.Choose(imgs);
                Color hue = Color.White;
                hue.G = GetHue(Position).G;
                hue.R = GetHue(Position).G;
                hue.B = GetHue(Position).G;
                if (color == CrystalColor.Rainbow)
                {
                }
                if (!SolidCheck(new Vector2(base.X - 4f, base.Y - 4f)))
                {
                    Add(new Image(img.GetSubtexture(0, 0, 14, 14)).SetOrigin(12f, 12f).SetColor(hue));
                }
                if (!SolidCheck(new Vector2(base.X + 4f, base.Y - 4f)))
                {
                    Add(new Image(img.GetSubtexture(10, 0, 14, 14)).SetOrigin(2f, 12f).SetColor(hue));
                }
                if (!SolidCheck(new Vector2(base.X + 4f, base.Y + 4f)))
                {
                    Add(new Image(img.GetSubtexture(10, 10, 14, 14)).SetOrigin(2f, 2f).SetColor(hue));
                }
                if (!SolidCheck(new Vector2(base.X - 4f, base.Y + 4f)))
                {
                    Add(new Image(img.GetSubtexture(0, 10, 14, 14)).SetOrigin(12f, 2f).SetColor(hue));
                }
                foreach (GCrystalStaticSpinner crystal in base.Scene.Tracker.GetEntities<GCrystalStaticSpinner>())
                {
                    if (crystal != this && crystal.AttachToSolid == AttachToSolid && crystal.X >= base.X && (crystal.Position - Position).Length() < 24f)
                    {
                        AddSprite((Position + crystal.Position) / 2f - Position);
                    }
                }
                base.Scene.Add(border = new Border(this, filler));
                expanded = true;
                Calc.PopRandom();
            }
        }

        private void AddSprite(Vector2 offset)
        {
            if (filler == null)
            {
                base.Scene.Add(filler = new Entity(Position));
                filler.Depth = base.Depth + 1;
            }
            List<MTexture> imgs = GFX.Game.GetAtlasSubtextures(
            "danger/crystal/bg_mine");
            Image spr = new Image(Calc.Random.Choose(imgs));
            spr.Position = offset;
            spr.Rotation = (float)Calc.Random.Choose(0, 1, 2, 3) * ((float)Math.PI / 2f);
            spr.CenterOrigin();

            spr.Color.G = GetHue(Position + offset).G;
            spr.Color.R = GetHue(Position + offset).G;
            spr.Color.B = GetHue(Position + offset).G;
            filler.Add(spr);
        }

        private bool SolidCheck(Vector2 position)
        {
            if (AttachToSolid)
            {
                return false;
            }
            foreach (Solid item in base.Scene.CollideAll<Solid>(position))
            {
                if (item is SolidTiles)
                {
                    return true;
                }
            }
            return false;
        }

        private void ClearSprites()
        {
            if (filler != null)
            {
                filler.RemoveSelf();
            }
            filler = null;
            if (border != null)
            {
                border.RemoveSelf();
            }
            border = null;
            foreach (Image item in base.Components.GetAll<Image>())
            {
                item.RemoveSelf();
            }
            expanded = false;
        }

        private void OnShake(Vector2 pos)
        {
            foreach (Component img in base.Components)
            {
                if (img is Image)
                {
                    (img as Image).Position = pos;
                }
            }
        }

        private bool IsRiding(Solid solid)
        {
            return CollideCheck(solid);
        }

        private void OnPlayer(Player player)
        {
            player.Die((player.Position - Position).SafeNormalize());
        }

        private void OnHoldable(Holdable h)
        {
            h.HitSpinner(this);
        }

        public override void Removed(Scene scene)
        {
            if (filler != null && filler.Scene == scene)
            {
                filler.RemoveSelf();
            }
            if (border != null && border.Scene == scene)
            {
                border.RemoveSelf();
            }
            base.Removed(scene);
        }

        public void Destroy(bool boss = false)
        {
            if (InView())
            {
                Audio.Play("event:/game/06_reflection/fall_spike_smash", Position);
                Color col = Color.Gray;
                if (color == CrystalColor.Red)
                {
                    col = Calc.HexToColor("ff4f4f");
                }
                else if (color == CrystalColor.Blue)
                {
                    col = Calc.HexToColor("639bff");
                }
                else if (color == CrystalColor.Purple)
                {
                    col = Calc.HexToColor("ff4fef");
                }
                CrystalDebris.Burst(Position, col, boss, 8);
            }
            RemoveSelf();
        }

        private Color GetHue(Vector2 position)
        {
            float dist = 280f;
            float percent = (position.Length() + base.Scene.TimeActive * 50f) % dist / dist;

            return Calc.HsvToColor(0.4f + Calc.YoYo(percent) * 0.4f, 0.4f, 0.9f);
        }
    }
}